package com.sun.beans.decoder;

import com.sun.beans.finder.ClassFinder;
import java.io.IOException;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParserFactory;
import java.security.PrivilegedAction;
import sun.misc.SharedSecrets;
import org.xml.sax.SAXException;
import org.xml.sax.Attributes;
import java.io.StringReader;
import org.xml.sax.InputSource;
import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.HashMap;
import java.security.AccessController;
import java.beans.ExceptionListener;
import java.lang.ref.Reference;
import java.util.List;
import java.util.Map;
import java.security.AccessControlContext;
import org.xml.sax.helpers.DefaultHandler;

public final class DocumentHandler extends DefaultHandler
{
    private final AccessControlContext acc;
    private final Map<String, Class<? extends ElementHandler>> handlers;
    private final Map<String, Object> environment;
    private final List<Object> objects;
    private Reference<ClassLoader> loader;
    private ExceptionListener listener;
    private Object owner;
    private ElementHandler handler;
    
    public DocumentHandler() {
        this.acc = AccessController.getContext();
        this.handlers = new HashMap();
        this.environment = new HashMap();
        this.objects = new ArrayList();
        this.setElementHandler("java", JavaElementHandler.class);
        this.setElementHandler("null", NullElementHandler.class);
        this.setElementHandler("array", ArrayElementHandler.class);
        this.setElementHandler("class", ClassElementHandler.class);
        this.setElementHandler("string", StringElementHandler.class);
        this.setElementHandler("object", ObjectElementHandler.class);
        this.setElementHandler("void", VoidElementHandler.class);
        this.setElementHandler("char", CharElementHandler.class);
        this.setElementHandler("byte", ByteElementHandler.class);
        this.setElementHandler("short", ShortElementHandler.class);
        this.setElementHandler("int", IntElementHandler.class);
        this.setElementHandler("long", LongElementHandler.class);
        this.setElementHandler("float", FloatElementHandler.class);
        this.setElementHandler("double", DoubleElementHandler.class);
        this.setElementHandler("boolean", BooleanElementHandler.class);
        this.setElementHandler("new", NewElementHandler.class);
        this.setElementHandler("var", VarElementHandler.class);
        this.setElementHandler("true", TrueElementHandler.class);
        this.setElementHandler("false", FalseElementHandler.class);
        this.setElementHandler("field", FieldElementHandler.class);
        this.setElementHandler("method", MethodElementHandler.class);
        this.setElementHandler("property", PropertyElementHandler.class);
    }
    
    public ClassLoader getClassLoader() {
        return (this.loader != null) ? ((ClassLoader)this.loader.get()) : null;
    }
    
    public void setClassLoader(ClassLoader referent) {
        this.loader = new WeakReference(referent);
    }
    
    public ExceptionListener getExceptionListener() {
        return this.listener;
    }
    
    public void setExceptionListener(ExceptionListener listener) {
        this.listener = listener;
    }
    
    public Object getOwner() {
        return this.owner;
    }
    
    public void setOwner(Object owner) {
        this.owner = owner;
    }
    
    public Class<? extends ElementHandler> getElementHandler(String str) {
        Class<Object> clazz = (Class)this.handlers.get(str);
        if (clazz == null) {
            throw new IllegalArgumentException("Unsupported element: " + str);
        }
        return clazz;
    }
    
    public void setElementHandler(String s, Class<? extends ElementHandler> clazz) {
        this.handlers.put(s, clazz);
    }
    
    public boolean hasVariable(String s) {
        return this.environment.containsKey(s);
    }
    
    public Object getVariable(String str) {
        if (!this.environment.containsKey(str)) {
            throw new IllegalArgumentException("Unbound variable: " + str);
        }
        return this.environment.get(str);
    }
    
    public void setVariable(String s, Object o) {
        this.environment.put(s, o);
    }
    
    public Object[] getObjects() {
        return this.objects.toArray();
    }
    
    void addObject(Object o) {
        this.objects.add(o);
    }
    
    @Override
    public InputSource resolveEntity(String s, String s2) {
        return new InputSource(new StringReader(""));
    }
    
    @Override
    public void startDocument() {
        this.objects.clear();
        this.handler = null;
    }
    
    @Override
    public void startElement(String s, String s2, String s3, Attributes attributes) throws SAXException {
        ElementHandler handler = this.handler;
        try {
            (this.handler = (ElementHandler)this.getElementHandler(s3).newInstance()).setOwner(this);
            this.handler.setParent(handler);
        }
        catch (Exception e) {
            throw new SAXException(e);
        }
        for (int i = 0; i < attributes.getLength(); ++i) {
            try {
                this.handler.addAttribute(attributes.getQName(i), attributes.getValue(i));
            }
            catch (RuntimeException ex) {
                this.handleException(ex);
            }
        }
        this.handler.startElement();
    }
    
    @Override
    public void endElement(String s, String s2, String s3) {
        try {
            this.handler.endElement();
        }
        catch (RuntimeException ex) {
            this.handleException(ex);
        }
        finally {
            this.handler = this.handler.getParent();
        }
    }
    
    @Override
    public void characters(char[] array, int n, int n2) {
        if (this.handler != null) {
            try {
                while (0 < n2--) {
                    this.handler.addCharacter(array[n++]);
                }
            }
            catch (RuntimeException ex) {
                this.handleException(ex);
            }
        }
    }
    
    public void handleException(Exception cause) {
        if (this.listener == null) {
            throw new IllegalStateException(cause);
        }
        this.listener.exceptionThrown(cause);
        return;
    }
    
    public void parse(final InputSource inputSource) {
        if (this.acc == null && null != System.getSecurityManager()) {
            throw new SecurityException("AccessControlContext is not set");
        }
        SharedSecrets.getJavaSecurityAccess().doIntersectionPrivilege(new PrivilegedAction<Void>() {
            @Override
            public Void run() {
                try {
                    SAXParserFactory.newInstance().newSAXParser().parse(inputSource, DocumentHandler.this);
                }
                catch (ParserConfigurationException ex) {
                    DocumentHandler.this.handleException(ex);
                }
                catch (SAXException ex2) {
                    Exception exception = ex2.getException();
                    if (exception == null) {
                        exception = ex2;
                    }
                    DocumentHandler.this.handleException(exception);
                }
                catch (IOException ex3) {
                    DocumentHandler.this.handleException(ex3);
                }
                return null;
            }
        }, AccessController.getContext(), this.acc);
        return;
    }
    
    public Class<?> findClass(String name) {
        try {
            return ClassFinder.resolveClass(name, this.getClassLoader());
        }
        catch (ClassNotFoundException ex) {
            this.handleException(ex);
            return null;
        }
    }
}
